@@ -1,10 +1,10 @@
 ############################################
-# 
-# Functions to transform a genind object 
+#
+# Functions to transform a genind object
 # into other R classes
 #
 # Thibaut Jombart
-# jombart@biomserv.univ-lyon1.fr
+# t.jombart@imperial.ac.uk
 #
 ############################################
 
@@ -17,16 +17,17 @@ genind2genotype <- function(x,pop=NULL,res.type=c("matrix","list")){
 
   if(!is.genind(x)) stop("x is not a valid genind object")
   if(x@ploidy != as.integer(2)) stop("not implemented for non-diploid genotypes")
+  checkType(x)
 
   if(!require(genetics)) stop("genetics package is not required but not installed.")
   if(is.null(pop)) pop <- x@pop
   if(is.null(pop)) pop <- as.factor(rep("P1",nrow(x@tab)))
   res.type <- tolower(res.type[1])
-  
+
   # make one table by locus from x@tab
   kX <- seploc(x,res.type="matrix")
   # kX is a list of nloc tables
-  
+
   # function to recode a genotype in form "A1/A2" from frequencies
   recod <- function(vec,lab){
     if(all(is.na(vec))) return(NA)
@@ -64,12 +65,14 @@ genind2genotype <- function(x,pop=NULL,res.type=c("matrix","list")){
     res <- cbind.data.frame(kGen)
     res <- makeGenotypes(res,convert=1:ncol(res))
   } else stop("Unknown res.type requested.")
-  
+
   return(res)
 }
 
 
 
+
+
 ############################
 # Function genind2hierfstat
 ############################
@@ -78,10 +81,11 @@ genind2hierfstat <- function(x,pop=NULL){
     ##   invisible(validObject(x))
     if(!is.genind(x)) stop("x is not a valid genind object")
     if(x@ploidy != as.integer(2)) stop("not implemented for non-diploid genotypes")
+    checkType(x)
 
     if(is.null(pop)) pop <- x@pop
     if(is.null(pop)) pop <- as.factor(rep("P1",nrow(x@tab)))
-    
+
     ## make one table by locus from x@tab
     kX <- seploc(x,res.type="matrix")
     ## kX is a list of nloc tables
@@ -96,7 +100,7 @@ genind2hierfstat <- function(x,pop=NULL){
     for(i in 1:nrmzero) {
         all.names <- lapply(all.names,function(e) gsub("^0","",e))
     }
-    
+
     ## function to recode a genotype in form "A1A2" (as integers) from frequencies
     recod <- function(vec,lab){
         if(all(is.na(vec))) return(NA)
@@ -124,17 +128,35 @@ genind2hierfstat <- function(x,pop=NULL){
 genind2df <- function(x, pop=NULL, sep="", usepop=TRUE){
 
   if(!is.genind(x)) stop("x is not a valid genind object")
+  ## checkType(x)
 
   if(is.null(pop)) {
       pop <- x@pop
       levels(pop) <- x@pop.names
   }
 
+  ## PA case ##
+  if(x@type=="PA"){
+      temp <- truenames(x)
+      if(is.list(temp) & usepop){
+          res <- cbind.data.frame(pop=temp[[2]],temp[[1]])
+      } else{
+          if(is.list(temp)) {
+              res <- temp[[1]]
+          } else{
+              res <- temp
+          }
+      }
+
+      return(res)
+  }
+
+  ## codom case ##
   # make one table by locus from x@tab
   kX <- seploc(x,res.type="matrix")
   kX <- lapply(kX, function(X) round(X*x@ploidy)) # take data as numbers of alleles
   ## (kX is a list of nloc tables)
-  
+
   ## function to recode a genotype in form "A1[sep]...[sep]Ak" from frequencies
   recod <- function(vec,lab){
       if(any(is.na(vec))) return(NA)
@@ -143,20 +165,6 @@ genind2df <- function(x, pop=NULL, sep="", usepop=TRUE){
   }
 
 
-  ## OLD VERSION
-  ##   recod <- function(vec,lab){
-  ##       vec <- as.logical(vec)
-  ##       sumVec <- sum(vec)
-  ##       if(is.na(sumVec)) {
-  ##           return(NA)
-  ##       } else if(sumVec==2){ # heteroZ
-  ##           return(paste(lab[vec], collapse=sep))
-  ##       } else if(sumVec==1){ # homoZ
-  ##           return(paste(lab[vec],lab[vec],sep=sep))
-  ##       } else return(NA)
-  ##   }
-  
-
   # kGen is a list of nloc vectors of genotypes
   kGen <- lapply(1:length(kX), function(i) apply(kX[[i]],1,recod,x@all.names[[i]]))
   names(kGen) <- x@loc.names
